@@ -2,8 +2,8 @@
 #
 # discan.R
 #
-# copyright (c) 2001-4, Karl W Broman, Johns Hopkins University
-# last modified Aug, 2004
+# copyright (c) 2001-5, Karl W Broman, Johns Hopkins University
+# last modified Apr, 2005
 # first written Oct, 2001
 # Licensed under the GNU General Public License version 2 (June, 1991)
 # 
@@ -20,34 +20,43 @@
 ######################################################################
 
 discan <-
-function(cross, pheno.col=1, method=c("em","mr"),
-         maxit=4000, tol=1e-4)
+function(cross, pheno, method=c("em","mr"),
+         addcovar=NULL, intcovar=NULL, maxit=4000, tol=1e-4,
+         verbose=FALSE)
 {
   method <- match.arg(method)
 
-  # check phenotypes
-  if(length(pheno.col) > 1) pheno.col <- pheno.col[1]
-  if(pheno.col < 1 || pheno.col > nphe(cross))
-    stop("Specified phenotype column is invalid.")
-
-  pheno <- cross$pheno[,pheno.col]
-  keep.ind <- (1:length(pheno))[!is.na(pheno)]
-  pheno <- pheno[keep.ind]
-  n.ind <- length(keep.ind)
+  n.ind <- nind(cross)
   n.chr <- nchr(cross)
   type <- class(cross)[1]
+  if(is.null(addcovar)) {
+    n.addcovar <- 0
+    addcovar <- 0
+  }
+  else n.addcovar <- ncol(addcovar)
+  if(is.null(intcovar)) {
+    n.intcovar <- 0
+    intcovar <- 0
+  }
+  else n.intcovar <- ncol(intcovar)
+
+  if(method=="mr" && n.addcovar+n.intcovar>0)  {
+    warning("Covariates ignored with method=\"mr\"; use \"em\" instead")
+    n.addcovar <- n.intcovar <- addcovar <- intcovar <- 0
+  }
 
   u <- unique(pheno)
-  if(any(u != 0 && u != 1))
+  if(any(u != 0 & u != 1))
     stop("Phenotypes must be either 0 or 1.")
 
-  if(method == "em") {
-    p <- mean(pheno)
-    n1 <- sum(pheno==1)
-    n0 <- sum(pheno==0)
-    if(n1==0 || n0==0) llik0 <- 0
-    else llik0 <- n1*log10(p) + n0*log10(1-p)
-  }
+  # get null log liklihood
+  if(n.addcovar > 0)
+    nullfit <- glm(pheno ~ addcovar, family=binomial(link=logit))
+  else
+    nullfit <- glm(pheno ~ 1, family=binomial(link=logit))
+  fitted <- nullfit$fitted
+  nullcoef <- nullfit$coef
+  llik0 <- sum(pheno*log10(fitted) + (1-pheno)*log10(1-fitted))
 
   results <- NULL
 
@@ -55,8 +64,20 @@ function(cross, pheno.col=1, method=c("em","mr"),
   for(i in 1:n.chr) {
 
     chrtype <- class(cross$geno[[i]])
-    if(chrtype=="X") sexpgm <- getsex(cross)
-    else sexpgm <- NULL
+    if(chrtype=="X") {
+      sexpgm <- getsex(cross)
+      ac <- revisecovar(sexpgm,addcovar)
+      n.ac <- ifelse(is.null(ac),0,ncol(ac))
+      ic <- revisecovar(sexpgm,intcovar)
+      n.ic <- ifelse(is.null(ic),0,ncol(ic))
+    }
+    else {
+      sexpgm <- NULL
+      ac <- addcovar
+      n.ac <- n.addcovar
+      ic <- intcovar
+      n.ic <- n.intcovar
+    }
 
     # get genotype names
     gen.names <- getgenonames(type,chrtype,"full",sexpgm)
@@ -65,9 +86,7 @@ function(cross, pheno.col=1, method=c("em","mr"),
     # pull out genotype data (mr)
     # or genotype probabilities (em)
     if(method == "mr") {
-      cfunc <- "R_discan_mr"
       newgeno <- cross$geno[[i]]$data
-      newgeno <- newgeno[keep.ind,]
       newgeno[is.na(newgeno)] <- 0 
 
       # discard partially informative genotypes
@@ -81,6 +100,16 @@ function(cross, pheno.col=1, method=c("em","mr"),
       n.pos <- ncol(newgeno)
       map <- cross$geno[[i]]$map
       if(is.matrix(map)) map <- map[1,]
+
+      z <- .C("R_discan_mr",
+              as.integer(n.ind),         # number of individuals
+              as.integer(n.pos),         # number of markers
+              as.integer(n.gen),         # number of possible genotypes
+              as.integer(newgeno),       # genotype data
+              as.integer(pheno),          # phenotype data
+              result=as.double(rep(0,n.pos*(n.gen+1))),
+              PACKAGE="qtl")
+
     }
     else {
       if(is.na(match("prob",names(cross$geno[[i]])))) { # need to run calc.genoprob
@@ -89,7 +118,6 @@ function(cross, pheno.col=1, method=c("em","mr"),
       }
       genoprob <- cross$geno[[i]]$prob
       n.pos <- ncol(genoprob)
-      genoprob <- genoprob[keep.ind,,]
 
       # revise X chromosome genotypes
       if(chrtype=="X" && (type=="bc" || type=="f2" || type=="f2ss"))
@@ -100,36 +128,54 @@ function(cross, pheno.col=1, method=c("em","mr"),
                         attr(cross$geno[[i]]$prob,"off.end"))
       if(is.matrix(map)) map <- map[1,]
 
-      cfunc <- "R_discan_im"
+      if(n.ac + n.ic > 0) {
+
+        start <- rep(nullcoef[1],n.gen)
+        if(n.ac > 0)
+          start <- c(start, nullcoef[-1])
+        if(n.ic > 0)
+          start <- c(start, rep(0, n.ic*(n.gen-1)))
+
+        z <- .C("R_discan_covar",
+                as.integer(n.ind),         # number of individuals
+                as.integer(n.pos),         # number of markers
+                as.integer(n.gen),         # number of possible genotypes
+                as.double(genoprob),       # genotype probabilities
+                as.double(ac),
+                as.integer(n.ac),
+                as.double(ic),
+                as.integer(n.ic),
+                as.integer(pheno),          # phenotype data
+                as.double(start),
+                result=as.double(rep(0,n.pos)),
+                as.integer(maxit),
+                as.double(tol),
+                as.integer(verbose),
+                PACKAGE="qtl")
+      }
+      else {
+        z <- .C("R_discan_im",
+                as.integer(n.ind),         # number of individuals
+                as.integer(n.pos),         # number of markers
+                as.integer(n.gen),         # number of possible genotypes
+                as.double(genoprob),       # genotype probabilities
+                as.integer(pheno),          # phenotype data
+                result=as.double(rep(0,n.pos*(n.gen+1))),
+                as.integer(maxit),
+                as.double(tol),
+                PACKAGE="qtl")
+      }
+
     }
-
-    # call the C function
-    if(method == "mr") 
-      z <- .C(cfunc,
-              as.integer(n.ind),         # number of individuals
-              as.integer(n.pos),         # number of markers
-              as.integer(n.gen),         # number of possible genotypes
-              as.integer(newgeno),       # genotype data
-              as.integer(pheno),          # phenotype data
-              result=as.double(rep(0,n.pos*(n.gen+1))),
-              PACKAGE="qtl")
-
-    else  # interval mapping
-      z <- .C(cfunc,
-              as.integer(n.ind),         # number of individuals
-              as.integer(n.pos),         # number of markers
-              as.integer(n.gen),         # number of possible genotypes
-              as.double(genoprob),       # genotype probabilities
-              as.integer(pheno),          # phenotype data
-              result=as.double(rep(0,n.pos*(n.gen+1))),
-              as.integer(maxit),
-              as.double(tol),
-              PACKAGE="qtl")
     z <- matrix(z$result,nrow=n.pos)
 
     if(method == "em") z[,1] <- z[,1] - llik0
     z[is.na(z[,1]),1] <- 0
-    colnames(z) <- c("lod",gen.names)
+
+    if(n.ac + n.ic > 0)
+      colnames(z) <- c("lod")
+    else
+      colnames(z) <- c("lod",gen.names)
       
     w <- names(map)
     o <- grep("^loc\-*[0-9]+",w)
@@ -141,8 +187,8 @@ function(cross, pheno.col=1, method=c("em","mr"),
     z <- cbind(chr=rep(names(cross$geno)[i],length(map)), pos=map, z)
     rownames(z) <- w
 
-
     # get null log10 likelihood for the X chromosome
+    adjustX <- FALSE
     if(chrtype=="X") {
 
       # determine which covariates belong in null hypothesis
@@ -153,19 +199,17 @@ function(cross, pheno.col=1, method=c("em","mr"),
       sexpgmcovar.alt <- temp$sexpgmcovar.alt      
 
       if(adjustX) { # get LOD-score adjustment
-        n.gen <- ncol(sexpgmcovar)+1
-
-        nullz <- .C("R_discan_mr",
-            as.integer(n.ind),
-            as.integer(1),
-            as.integer(n.gen),
-            as.integer(sexpgmcovar.alt),
-            as.integer(pheno),
-            result=as.double(rep(0,n.gen+1)),
-            PACKAGE="qtl")
+        if(n.ac > 0) 
+          nullfit <- glm(pheno ~ ac+sexpgmcovar,
+                         family=binomial(link=logit))
+        else 
+          nullfit <- glm(pheno ~ sexpgmcovar,
+                         family=binomial(link=logit))
+        fitted <- nullfit$fitted
+        llik0X <- sum(pheno*log10(fitted) + (1-pheno)*log10(1-fitted))
 
         # adjust LOD curve
-        z[,3] <- z[,3] - nullz$result[1]
+        z[,3] <- z[,3] - (llik0X - llik0)
       }
     } 
 
@@ -212,16 +256,21 @@ function(cross, pheno.col=1, method=c("em","mr"),
     }
 
     results <- rbind(results, z)
-  }
+  } # loop over chromosomes
 
-  # sort the later columns
-  neworder <- c(colnames(results)[1:3],sort(colnames(results)[-(1:3)]))
-  results <- results[,neworder]
+  if(ncol(results) > 3) {
+    # sort the later columns
+    neworder <- c(colnames(results)[1:3],sort(colnames(results)[-(1:3)]))
+    results <- results[,neworder]
+  }
 
   class(results) <- c("scanone","data.frame")
   attr(results,"method") <- method
   attr(results,"type") <- type
   attr(results,"model") <- "binary"
+  attr(results,"null.log10.lik") <- llik0
+  if(adjustX) 
+    attr(results,"null.log10.lik.X") <- llik0X
   results
 }
 
